@@ -1,62 +1,61 @@
 # -*- coding: utf-8 -*
-import itertools
-from lca import LCA
+from __future__ import division
+from .lca import LCA
 from scipy.sparse.linalg import iterative, spsolve
 from stats_toolkit.random import MCRandomNumberGenerator
+import itertools
 import multiprocessing
-import numpy as np
 
 
 class MonteCarloLCA(LCA):
     def __init__(self, demand, method=None, iter_solver=iterative.cgs,
-            seed=None, *args, **kwargs):
+                 seed=None, *args, **kwargs):
         super(MonteCarloLCA, self).__init__(demand, method=method, *args,
-            **kwargs)
+                                            **kwargs)
         self.seed = seed
         self.iter_solver = iter_solver
         self.guess = None
-        self.load_databases()
-        self.load_method()
+        self.load_lci_data()
+        self.load_lcia_data()
         self.tech_rng = MCRandomNumberGenerator(self.tech_params, seed=seed)
         self.bio_rng = MCRandomNumberGenerator(self.bio_params, seed=seed)
         self.cf_rng = MCRandomNumberGenerator(self.cf_params, seed=seed)
 
+    def __iter__(self):
+        return self
+
     def next(self):
-        self.build_technosphere_matrix(self.tech_rng.next())
-        self.build_biosphere_matrix(self.bio_rng.next())
-        self.build_characterization_matrix(self.cf_rng.next())
+        self.rebuild_technosphere_matrix(self.tech_rng.next())
+        self.rebuild_biosphere_matrix(self.bio_rng.next())
+        self.rebuild_characterization_matrix(self.cf_rng.next())
 
         if not hasattr(self, "demand_array"):
             self.build_demand_array()
 
         self.lci_calculation()
         self.lcia_calculation()
+        return self.score
 
     def solve_linear_system(self):
-        if not self.iter_solver or self.guess == None:
+        if not self.iter_solver or self.guess is None:
             self.guess = spsolve(
-                self.technosphere_matrix.data,
-                self.demand_array.data)
+                self.technosphere_matrix,
+                self.demand_array)
             return self.guess
         else:
             solution, status = self.iter_solver(
-                self.technosphere_matrix.data,
-                self.demand_array.data,
+                self.technosphere_matrix,
+                self.demand_array,
                 x0=self.guess)
             if status != 0:
                 raise
             return solution
 
-    def iterate(self):
-        # TODO: use yield or __iter__? What is most pythonic?
-        self.next()
-        return self.score
-
 
 class ComparativeMonteCarlo(LCA):
     """First draft approach at comparative LCA"""
     def __init__(self, demands, method=None, iter_solver=iterative.cgs,
-            seed=None, *args, **kwargs):
+                 seed=None, *args, **kwargs):
         self.demands = demands
         # Get all possibilities for database retrieval
         demand_all = demands[0].copy()
@@ -66,16 +65,20 @@ class ComparativeMonteCarlo(LCA):
         self.seed = seed
         self.iter_solver = iter_solver
         self.guess = None
-        self.load_databases()
-        self.load_method()
+        self.load_lci_data()
+        self.load_lcia_data()
         self.tech_rng = MCRandomNumberGenerator(self.tech_params, seed=seed)
         self.bio_rng = MCRandomNumberGenerator(self.bio_params, seed=seed)
         self.cf_rng = MCRandomNumberGenerator(self.cf_params, seed=seed)
 
+    def __iter__(self):
+        return self
+
     def next(self):
-        self.build_technosphere_matrix(self.tech_rng.next())
-        self.build_biosphere_matrix(self.bio_rng.next())
-        self.build_characterization_matrix(self.cf_rng.next())
+        self.rebuild_technosphere_matrix(self.tech_rng.next())
+        self.rebuild_biosphere_matrix(self.bio_rng.next())
+        self.rebuild_characterization_matrix(self.cf_rng.next())
+
         results = []
         for demand in self.demands:
             self.build_demand_array(demand)
@@ -85,15 +88,15 @@ class ComparativeMonteCarlo(LCA):
         return results
 
     def solve_linear_system(self):
-        if not self.iter_solver or self.guess == None:
+        if not self.iter_solver or self.guess is None:
             self.guess = spsolve(
-                self.technosphere_matrix.data,
-                self.demand_array.data)
+                self.technosphere_matrix,
+                self.demand_array)
             return self.guess
         else:
             solution, status = self.iter_solver(
-                self.technosphere_matrix.data,
-                self.demand_array.data,
+                self.technosphere_matrix,
+                self.demand_array,
                 x0=self.guess)
             if status != 0:
                 raise
@@ -105,7 +108,8 @@ class ComparativeMonteCarlo(LCA):
 
 class ParallelMonteCarlo(object):
     """Split a Monte Carlo calculation into parallel jobs"""
-    def __init__(self, demand, method, iterations=1000, chunk_size=None):
+    def __init__(self, demand, method, iterations=1000, chunk_size=None,
+                 cpus=None):
         self.demand = demand
         self.method = method
         if chunk_size:
@@ -114,13 +118,14 @@ class ParallelMonteCarlo(object):
             if iterations % self.chunk_size:
                 self.num_jobs += 1
         else:
-            self.num_jobs = max(multiprocessing.cpu_count() - 1, 1)
+            self.num_jobs = cpus or multiprocessing.cpu_count()
             self.chunk_size = (iterations // self.num_jobs) + 1
 
     def calculate(self):
-        pool = multiprocessing.Pool(processes=max(multiprocessing.cpu_count() - 1, 1))
+        pool = multiprocessing.Pool(processes=max(
+            multiprocessing.cpu_count() - 1, 1))
         results = [pool.apply_async(single_worker, (self.demand, self.method,
-            self.chunk_size)) for x in xrange(self.num_jobs)]
+                   self.chunk_size)) for x in xrange(self.num_jobs)]
         pool.close()
         pool.join()  # Blocks until calculation is finished
         return list(itertools.chain(*[x.get() for x in results]))
@@ -146,7 +151,7 @@ each Monte Carlo iteration.
     def calculate(self):
         pool = multiprocessing.Pool(processes=multiprocessing.cpu_count() - 1)
         results = [pool.apply_async(multi_worker, (self.demands, self.method)
-            ) for x in xrange(self.iterations)]
+                                    ) for x in xrange(self.iterations)]
         pool.close()
         pool.join()  # Blocks until calculation is finished
         return self.merge_dictionaries(*[x.get() for x in results])
@@ -160,12 +165,15 @@ def single_worker(demand, method, iterations):
 
 def multi_worker(demands, method):
     lca = LCA(demands[0], method)
-    lca.load_databases()
-    lca.load_method()
+    lca.load_lci_data()
+    lca.load_lcia_data()
     # Create new matrices
-    lca.build_technosphere_matrix(MCRandomNumberGenerator(lca.tech_params).next())
-    lca.build_biosphere_matrix(MCRandomNumberGenerator(lca.bio_params).next())
-    lca.build_characterization_matrix(MCRandomNumberGenerator(lca.cf_params).next())
+    lca.rebuild_technosphere_matrix(
+        MCRandomNumberGenerator(lca.tech_params).next())
+    lca.rebuild_biosphere_matrix(
+        MCRandomNumberGenerator(lca.bio_params).next())
+    lca.rebuild_characterization_matrix(
+        MCRandomNumberGenerator(lca.cf_params).next())
     lca.decompose_technosphere()
     lca.lci()
     lca.lcia()
